// TARGET:
//[Chords]
//|F / / / |C / / / |Bb / / / |C / / / |F / / / |C / / / |Bb / / / |C / / / |
//|Dm / / / |C / / / |Bb / / / |Gm / / / |Dm / / / |Bb / / / |F / / / |C / / / |
//|F / / / | / / / / | / / / / | / / / / |Gm / / / | / / / / | / / / / | / / / / |
//|Bb / / / | / / / / | / / / / | / / / / |F / / / | / / / / | Gm / / /| / / / / |
//|F| / / / / | / / / / | / / / / |
//[ChordsEnd]
//
// NOTE: Script command appears under "Event" item in main menu bar; To dump all chords,
// select the first chord on the chord track before invoking this script command.
function print(vValue) {
  Host.Console.writeLine(vValue.toString())
}


function ChordsToBiabTextFile() {
  // Setup
  this.interfaces = [Host.Interfaces.IEditTask];
  this.prepareEdit = function(context) {
   return Host.Results.kResultOk;
  }

  // Host command shortcut
  var cmd = Host.GUI.Commands;

  // Select all chord events
  cmd.interpretCommand("Edit", "Select All");

  this.performEdit = function(context) {

    // Array to hold the chord events
    var chords = [];

    // Collect selected chords into an array
    var iterator = context.iterator;
    while (!iterator.done()) {
      var chordEvent = iterator.next();
      var it = chordEvent.createIterator();

      // Ensure that first chord was selected...
      
      if (chordEvent.mediaType != undefined) {
        Host.GUI.alert("Please select the first chord on timeline.")
        return;
      }

      var chordName = chordEvent.name; // Why always empty?

      var startTime = chordEvent.startTime.string;  // "0001.01.01.00"   BAR.BEAT.SUBBEAT.HUNDREDTHS
      var endTime = chordEvent.endTime.string;  // "0001.01.01.00"   BAR.BEAT.SUBBEAT.HUNDREDTHS
      var timeParts = {"startTime": startTime.split("."), "endTime": endTime.split(".")};

      var data = {
        "event": chordEvent,
        "name": chordName,
        "startTime": { "time": startTime, "bar": Number(timeParts["startTime"][0]), "beat": Number(timeParts["startTime"][1]), "subBeat": Number(timeParts["startTime"][2]), "hundredths": Number(timeParts["startTime"][3]) },
        "endTime": { "time": endTime, "bar": Number(timeParts["endTime"][0]), "beat": Number(timeParts["endTime"][1]), "subBeat": Number(timeParts["endTime"][2]), "hundredths": Number(timeParts["endTime"][3]) }
      };

      chords.push(data);
    }

    // Sort chords by start time
    chords.sort(function(a, b) {
      var comp1 = a.startTime.time;
      var comp2 = b.startTime.time;
      return (comp1 < comp2) ? -1 : ((comp1 > comp2) ? 1 : 0);
    });

    // De-select all chord events
    cmd.interpretCommand("Edit", "Deselect All");

    // Assign chord names to each chord object
    cmd.interpretCommand('Navigation', 'First Event'); // Select the first event
    for (i = 0; i < chords.length; ++i) {
      var chordSelector = Host.Objects.getObjectByUrl('object://hostapp/DocumentManager/ActiveDocument/EventInspector/EventInfo/ChordSelector');
      //var chordSelector = Host.Objects.getObjectByUrl('object://hostapp/DocumentManager/ActiveDocument/TrackList/CurrentChord/EventInfo/ChordSelector');
      chords[i].name = chordSelector.findParameter('chord').string;
      cmd.interpretCommand('Navigation', 'Next Event'); // Advance to next event
    }

    var fileSelector = Host.Classes.createInstance("CCL:FileSelector");
    var textType = {
      filename: ("*.txt"),
      description: ("Song.txt"),
      extension: "txt",
      mimetype: "text"
    };
    fileSelector.addFilter(textType);
    fileSelector.runSave();

    // Store the user selected path
    var path = fileSelector.getPath();

    var file = Host.IO.createTextFile(path);
    if (file) {
      let nl = endLine(); // use platform line ending
      var textBuffer = [];

      textBuffer.push("[Chords]" + nl);
      textBuffer.push("|");

      // Determine the number to use for beats per bar
      var beatsPerBar = 3;
      var beatsPerBarInput = Host.GUI.ask("Use 4 beats per bar instead of 3 ?");
      if (beatsPerBarInput == Host.GUI.Constants.kYes) {
        beatsPerBar = 4;
      }

      var endChord = chords[chords.length - 1];
      var finalBeat = absoluteBeatPosition(endChord.endTime.bar, endChord.endTime.beat, beatsPerBar) - 1;
      var previousChordName = null;
      var i;
      for (i = 1; i <= finalBeat; ++i) {
        var chord = findActiveChord(chords, i, beatsPerBar);
        if (null != chord) {
          // Get the current chord name

          var chordName = chord.name;
          if (chordName != previousChordName) {
            // Print the new chord name
            chordName1 = chordName.replace(/\u266f/, '#'); 
            chordName1 = chordName1.replace(/\u266d/, 'b');
            textBuffer.push(chordName1); 
            previousChordName = chordName;
          } 
          else {
            // Repeat previous beat's chord
            textBuffer.push("/");
          }
        }
        else {
          if (null == previousChordName) {
            // No chord defined yet
            textBuffer.push("-");
          }
          else {
            // Repeat previous beat's chord
            textBuffer.push("/");
          }
        }
        if (i % beatsPerBar == 0) {
          // Close off a bar
          textBuffer.push("|");
          if (i % (beatsPerBar * 8) == 0 && i != finalBeat) {
            // Add a new line after every 8 bars (except if last beat)
            
            textBuffer.push(nl);
            textBuffer.push("|");
          }
        }
        else {
          // Add whitespace between beats
          textBuffer.push(" ");
        }
      }
      textBuffer.push(nl);
      textBuffer.push("[ChordsEnd]");

      var text = textBuffer.join(""); //Concatenate all entries into one block of text


      file.writeLine(text);
      file.close();
      // move to the first chord event
      cmd.interpretCommand("Navigation", "First Event")    
      Host.GUI.alert("Chords written to text file");
      // print to message window 
      print("Chords copied to txt file");     
    }

    return Host.Results.kResultOk;
  }
}

function findActiveChord(chords, absoluteBeatNumber, beatsPerBar) {
  var retVal = null;
  var i;

  // Scan for chord that starts on the given absolute beat
  for (i = 0; i < chords.length; ++i) {
    var index = i;
    var chordStart = chords[index].startTime;
    var chordEnd = chords[index].endTime;
    var beatStart = absoluteBeatPosition(chordStart.bar, chordStart.beat, beatsPerBar);
    var beatEnd = absoluteBeatPosition(chordEnd.bar, chordEnd.beat, beatsPerBar);
    if (beatStart == absoluteBeatNumber) {
      retVal = chords[index];
      break;
    }
  }

  if (retVal == null) {
    // Scan for chord that was last active
    for (i = 0; i < chords.length; ++i) {
      var index = i;
      var chordStart = chords[index].startTime;
      var chordEnd = chords[index].endTime;
      var beatStart = absoluteBeatPosition(chordStart.bar, chordStart.beat, beatsPerBar);
      var beatEnd = absoluteBeatPosition(chordEnd.bar, chordEnd.beat, beatsPerBar);
      if (beatStart < absoluteBeatNumber && beatEnd > absoluteBeatNumber) {
        retVal = chords[index];
        break;
      }
    }
  }

  return retVal;
}

function absoluteBeatPosition(bar, beat, beatsPerBar) {
  return ((bar - 1) * beatsPerBar) + beat;
}

/** Get platform line ending. */
function endLine() {
  var retVal;
  var platform = Host.getPlatform();
  switch (platform) {
    case "win":
      retVal = "\r\n";
      break;
    case "mac":
      retVal = "\r";
      break;
    default:
      retVal = "\n";
  }
  return retVal;
}

function printObjectToConsole(vValue) {
  Host.Console.writeLine(JSON.stringify(vValue));
}

function createInstance() {
  return new ChordsToBiabTextFile;
}